/*
 * Copyright 2007 Niels Provos <provos@citi.umich.edu>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _EVUTILS_H_
#define _EVUTILS_H_

#include <stdint.h>

/* keep track of events as they happen */

#define EVENT_SLOTS	60	/* for seconds and minutes */

struct event_single_slot {
	uint32_t count;
	struct timeval tv;
};

struct event_slot {
	struct event_single_slot slots[EVENT_SLOTS];
	int cur_pos;
	
	struct timeval interval;
	struct timeval threshold;	/* EVENT_SLOTS * interval */
};

struct event_counter {
	uint32_t total;

	char *name;

	int num_slots;
	struct event_slot *slots;
};

struct event_counter *event_counter_new(const char *name,
    int num_entries, struct timeval *intervals);
void event_counter_free(struct event_counter *ctr);
void event_counter_add(struct event_counter *ctr, uint32_t count);

/* returns num_entry counts */
void event_counter_get(struct event_counter *ctr, uint32_t *counts);

/* a histogram of named stats */
struct event_histogram {
	int num_slots;
	int used_slots;
	struct event_counter **counters;
};

struct event_histogram *event_histogram_new(int num_slots);
void event_histogram_add(struct event_histogram *ehg,
    const char *name, uint32_t count);
void event_histogram_as_html(struct evbuffer *buf, const char *css_class,
    struct event_histogram *ehg);

/* misc utility functions */
void event_seconds_as_eta(time_t seconds, char *output, size_t out_len);

/* simple timestamped buffer of histories */

struct history_entry {
	TAILQ_ENTRY(history_entry) (next);

	struct timeval tv;
	struct evbuffer *ev_buf;
};

struct history_buffer {
	TAILQ_HEAD(historyq, history_entry) (entries);
	int num_entries;
	int max_entries;
};

struct history_buffer* history_buffer_new(int max_entries);
void history_enter_buffer(struct history_buffer *hist, struct evbuffer *buf);
void history_to_html(struct evbuffer *output, struct history_buffer *hist);

#endif /* _EVUTILS_H_ */
